<?php

class Questions {
    
    /**
     * fetch a set of questions with respective answers
     * @global type $dbh
     * @return type
     */
    static function get_set_of_questions()
    {
        //if a final result exists already in session
        if (isset($_SESSION['final_result']))
        {
            //return it
            return $_SESSION['final_result'];
        }//if (isset($_SESSION['final_result']))
        
        global $dbh;
        
        //init vars
        $row = array();
        $final_result = array();
        $categories = array();
        $category_active = 1;
        
        //query to get all active categories
        $query_get_categories = '
            SELECT
                *
            FROM
                ' . TABLE_PREFIX . 'question_categories
            WHERE
                category_active = :status
        ';
        
        //query to get one random question for category
        $query_get_question_for_category = '
            SELECT 
                *
            FROM
                ' . TABLE_PREFIX . 'questions
            WHERE
                question_category = :question_category
            ORDER BY
                RAND()
            LIMIT
                10
        ';
        
        //query to get all answers for one question
        $query_get_answers_for_question = '
            SELECT
                *
            FROM
                ' . TABLE_PREFIX . 'answers
            WHERE
                answer_of_question = :question_id
        ';
        
        $stmt_get_categories = $dbh->prepare($query_get_categories);
        $stmt_get_question_for_category = $dbh->prepare($query_get_question_for_category);
        $stmt_get_answers_for_question = $dbh->prepare($query_get_answers_for_question);
        
        try
        {
            $stmt_get_categories->bindParam(':status', $category_active);
            $stmt_get_categories->execute();
            
            //parse results from db
            while ($row = $stmt_get_categories->fetch())
            {
                //add current rows to categories array
                $categories[] = $row;
            }//while ($row = $stmt_get_categories->fetch())
            
            //init counter
            $count = 1;
            //parse categories
            foreach($categories as $category)
            {
                $stmt_get_question_for_category->bindParam(':question_category', $category['category_id']);
                $stmt_get_question_for_category->execute();
                
                //get question
                while($row = $stmt_get_question_for_category->fetch())
                {
                    //add to result details of question
                    $final_result[$count]['question_label'] = $row['question_label'];
                    $final_result[$count]['question_category'] = $category['category_name'];
                    $final_result[$count]['question_id'] = $row['question_id'];

                    $stmt_get_answers_for_question->bindParam(':question_id', $row['question_id']);
                    $stmt_get_answers_for_question->execute();

                    //parse all answers for question
                    while ($answer = $stmt_get_answers_for_question->fetch())
                    {
                        $final_result[$count]['answers'][$answer['answer_id']]['text'] = $answer['answer_text'];
                        $final_result[$count]['answers'][$answer['answer_id']]['value'] = $answer['answer_value'];
                    }//while ($answer = $stmt_get_answers_for_question->fetch())
                    $count++;
                }//while($row = $stmt_get_question_for_category->fetch())
            }//foreach($categories as $category)
        }//try
        catch(PDOException $e)
        {
            Utils::sql_fail($query_get_answers_for_question, $e->getMessage());
        }//catch(PDOException $e)
        
        //add result to session
        $_SESSION['final_result'] = $final_result;
        
        return $final_result;
    }//end function get_all_questions
    
    /**
     * get all scores from database
     * @global type $dbh
     * @return type
     */
    static function get_all_scores()
    {
        global $dbh;
        $result = array();
        
        //query to get all scores
        $query = '
            SELECT
                *
            FROM
                ' . TABLE_PREFIX . 'scores
            ORDER BY
                score_id
        ';
        
        try
        {
            $stmt = $dbh->query($query);
        }//try
        catch(PDOException $e)
        {
            Utils::sql_fail($query, $e->getMessage());
        }//catch(PDOException $e)
        
        //parse results from db
        while ($row = $stmt->fetch())
        {
            $result[$row['score_value']] = array(
                'score_title' => $row['score_title'], 
                'score_tagline1' => $row['score_tagline1'], 
                'score_tagline2' => $row['score_tagline2'], 
                'score_left' => $row['score_left']);
        }//while ($row = $stmt->fetch())
        
        //return result
        return $result;
    }//end function get_all_scores
}

