<?php

class Utils 
{
    /**
     * redirect()
     * redirects the user to a certain url within the website (WWW_ROT . $url_extra);
     * @param string $url_extra
     * @param integer $nr_seconds
     * @param boolean $on_ssl
     * @return void 
     */
    static function redirect($url_extra = '', $nr_seconds = 0, $on_ssl = false) 
	{
        // default base url: WWW_ROOT 
        $root = WWW_ROOT;
		// secure connection.. use 
        if ($on_ssl) 
		{
            $root = SECURE_ROOT;
        }
		
		//// add a delay before redirect 
        if ($nr_seconds) 
		{ 
            header('refresh: ' . $nr_seconds . '; url=' . $root . $url_extra);
        } 
		else // redirect now 
		{
            header("HTTP/1.1 301 Moved Permanently");
            header('Location: ' . $root . $url_extra);
        }
        die;
    }// end function redirect();
	
    /**
     * handle sql errors 
     */
    static function sql_fail($query, $error_message = '') 
	{
        _pre($query);
        _pre($error_message);
        die;
    }//end function sql_fail
	
    /**
     * general refresh function.. useful after a form is submitted via post to avoid resending the values on refresh
     */
    static function refresh_page() 
	{
        header("HTTP/1.1 301 Moved Permanently");
        header('Location: http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
        die;
    }//end function refresh_page
	
    /**
     * calculate user score, depending on the answers
     * @global type $questions
     * @param type $answers
     * @return type
     */
	static function get_user_score($answers)
	{
		global $questions;
        //init score
		$nota = 0;
        
        //parse users answers
		foreach ($answers as $question_id=>$answer_id)
		{
            //if chosen answer exists
			if (isset($questions[$question_id]['answers'][$answer_id]))
			{
                //if chosen answer is the right one (value not 0)
				if ($questions[$question_id]['answers'][$answer_id]['value'])
				{
                    //increment score
					$nota++;
				}//if ($questions[$question_id]['answers'][$answer_id]['value'])
			}//if (isset($questions[$question_id]['answers'][$answer_id]))
		}//foreach ($answers as $question_id=>$answer_id)
        
        //apply formula for final score
        $nota = round(($nota/count($questions))*10);
        
        //return score
		return $nota;
	}//function get_user_score
	
    //create and upload the diploma
	static function render_user_pic($nota,  $nume)
	{
        //generate the diploma
		$image = self::create_image($nume, $nota);
        //upload diploma in user galery
		self::upload_to_fb($image,$nume);
        //delete the diploma from server
		@unlink(SITE_ROOT . 'images/fb-images/' . $image . '.jpg');
	}//function render_user_pic
	
    /**
     * upload diploma to user gallery
     * @global type $user_profile
     * @global type $facebook
     * @global type $fbconfig
     * @param type $image
     * @param type $name
     */
	static function upload_to_fb($image, $name)
	{
            global $user_profile, $facebook, $fbconfig;
            
            //create array of photo details:
            // - message is the description of photo
            $photo_details = array(
                    'message'=> $name .' fount out how well knows PHP. Find out too! Click here: '. $fbconfig['appBaseUrl']
            );

            //path of diploma
            $photo_details['image'] = '@' . SITE_ROOT . 'images/fb-images/' . $image . '.jpg';

            //upload photo to user gallery using the facebook api
            $upload_photo = $facebook->api('/me/photos', 'post', $photo_details);
	}//function upload_to_fb
	
    /**
     * save the user details
     * @global type $dbh
     * @global type $request_timestamp
     * @param type $user_name
     * @param type $user_email
     * @param type $answers
     * @param type $grade
     * @param type $fb_id
     * @param type $location
     */
    static function save_user_details($user_name, $user_email, $answers, $grade,
            $fb_id, $location)
    {
        global $dbh, $request_timestamp;
        //get last entry in answers array
        $end = end($answers);
        //get the time it took to complete the quiz
        $total_time = $end['time'] - $answers[0]['time'];
        
        //query to insert the new user details in db
        $query = '
            INSERT INTO
                ' . TABLE_PREFIX . 'users
            SET
                user_name = :user_name,
                user_email = :user_email,
                user_date = :date,
                user_grade = :user_grade,
                user_fb_id = :user_fb_id,
                user_location = :user_location,
                user_total_time = :user_total_time
        ';

        $stmt = $dbh->prepare($query);

        try
        {
            $stmt->bindParam(':user_name', $user_name);
            $stmt->bindParam(':user_email', $user_email);
            $stmt->bindParam(':date', $request_timestamp);
            $stmt->bindParam(':user_grade', $grade);
            $stmt->bindParam(':user_fb_id', $fb_id);
            $stmt->bindParam(':user_location', $location);
            $stmt->bindParam(':user_total_time', $total_time);
            $stmt->execute();
        }//try
        catch(PDOException $e)
        {
            Utils::sql_fail($query, $e->getMessage());
        }//catch(PDOException $e)
        
        //save answers of user also
        self::save_user_answers($dbh->lastInsertId(), $answers);
    }//function save_user_details
        
    /**
     * save the answers of user
     * @global type $dbh
     * @param type $user_id
     * @param type $answers
     */
    static function save_user_answers($user_id, $answers)
    {
        global $dbh;
        
        //query to insert answer of user in db
        $query = '
            INSERT INTO
                ' . TABLE_PREFIX . 'user_answers
            SET
                user_answer_user = :user_id,
                user_answer_question = :question_id,
                user_answer_value = :answer_id,
                user_answer_time = :answer_time
        ';
        
        $stmt = $dbh->prepare($query);
        
        try
        {
            //parse the answers
            foreach($answers as $key => $val)
            {
                $stmt->bindValue(':user_id', $user_id);
                $stmt->bindValue(':question_id', $key);
                $stmt->bindValue(':answer_id', $val['value']);
                $stmt->bindValue(':answer_time', $val['time']);
                $stmt->execute();
            }//foreach($answers as $key => $val)
        }//try
        catch(PDOException $e)
        {
            Utils::sql_fail($query, $e->getMessage());
        }//catch(PDOException $e)
    }//function save_user_answers
	
    /**
     * create the diploma
     * @param type $name
     * @param type $grade
     * @return type
     */
	static function create_image($name, $grade)	
	{
        //create gd resource
        //!this image must exists, it's the background of the diploma
        $image = imagecreatefromjpeg(SITE_ROOT . 'images/diploma-bg.JPG');

        //create colors for user name and user score
        $name_color = imagecolorallocate($image, 168, 174, 197); 
        $grade_color = imagecolorallocate($image, 255, 255, 255); 

        // ----- TO FILL IN ----- must replace with your font name
        $font = SITE_ROOT . 'font/your-font-name.ttf';

        //add username to diploma
        imagettftext($image, 36, 0, 125, 479, $name_color, $font, $name);

        //if grade is lower than 10 (has only 1 digit)
        if(10 > $grade)
        {
            //add grade on diploma to coordinates to fit a one digit number
            imagettftext($image, 100, 0, 656, 360, $grade_color, $font, $grade);
        }//if(10 > $grade)
        else
        {
            //add grade on diploma to coordinates to fit a two digit number
            imagefttext($image, 100, 0, 608, 360, $grade_color, $font, $grade);
        }//else -> if(10 > $grade)

        //Output the newly created image in jpeg format 
        imagejpeg($image, SITE_ROOT . 'images/fb-images/'.md5($name.$grade).'.jpg');   
        //Free up resources
        imagedestroy($image);

        return md5($name.$grade);
	}//function create_image
	
    //extract the right answer from question
	static function extract_right_answer_text($answers)
	{
        //parse answers array
		foreach ($answers as $key=>$value)
		{
            //if current answer value is not 0
			if ($value['value'])
			{
                //return the text of the answer
				return $value['text'];
			}//if ($value['value'])
		}//foreach ($answers as $key=>$value)
		return 'N/A';
	}//function extract_right_answer_text
	
}// end class Utils